from picamera.array import PiRGBArray
from picamera import PiCamera
from imutils import contours
import time
import imutils
import cv2
import numpy as np


def order_points(pts): #funzione per four_point_transform

	rect = np.zeros((4, 2), dtype = "float32")

	s = pts.sum(axis = 1)
	rect[0] = pts[np.argmin(s)]
	rect[2] = pts[np.argmax(s)]

	diff = np.diff(pts, axis = 1)
	rect[1] = pts[np.argmin(diff)]
	rect[3] = pts[np.argmax(diff)]

	return rect

def four_point_transform(image, pts): #funzione che "aggiusta" la prospettiva dell'immagine

	rect = order_points(pts)
	(tl, tr, br, bl) = rect

	widthA = np.sqrt(((br[0] - bl[0]) ** 2) + ((br[1] - bl[1]) ** 2))
	widthB = np.sqrt(((tr[0] - tl[0]) ** 2) + ((tr[1] - tl[1]) ** 2))
	maxWidth = max(int(widthA), int(widthB))

	heightA = np.sqrt(((tr[0] - br[0]) ** 2) + ((tr[1] - br[1]) ** 2))
	heightB = np.sqrt(((tl[0] - bl[0]) ** 2) + ((tl[1] - bl[1]) ** 2))
	maxHeight = max(int(heightA), int(heightB))

	dst = np.array([
		[0, 0],
		[maxWidth - 1, 0],
		[maxWidth - 1, maxHeight - 1],
		[0, maxHeight - 1]], dtype = "float32")

	M = cv2.getPerspectiveTransform(rect, dst)
	warped = cv2.warpPerspective(image, M, (maxWidth, maxHeight))

	return warped

def riconosci_parte(img):
    #cerca di identificare un cerchio nell'immagine (nel caso nella cella ci fosse un O) (valutare la modifica dei parametri della funzione per un riconoscimento più preciso)
	circles=cv2.HoughCircles(img,cv2.cv.CV_HOUGH_GRADIENT,1,25,param1=55,param2=25,minRadius=10,maxRadius=0)
    if circles is not None: #se nell'immagine c'è effettivamente un cerchio allora si suppone ci sia il simbolo O nella cella
		return "O"
    elif np.sum(img == 0) > np.sum(img < 256) * 0.15: #se nell'immagine non c'è un cerchio allora si verifica se ci sia un altro simbolo (X), sapendo che con i filtri applicati in precedenza alle immagini si hanno (se presenti) le pedine contenenti i simboli di colore nero su uno sfondo bianco (la cella) allora "contando" i pixel neri nell'immagine della cella e verificando che siano almeno il 15% (per evitare errori) rispetto al totale dei pixel possiamo affermare che nella cella ci sia una pedina, e non essendo un O (già teoricamente rillevato) è una X
		return "X"
	else:
        #la cella è vuota
		return " "

def get_board():

    #inizializzanzione fotocamera
	camera = PiCamera()
	camera.resolution = (640, 480)
	rawCapture = PiRGBArray(camera)

	time.sleep(0.1)
    
    #scatto foto
	camera.capture(rawCapture, format="bgr")
	img = rawCapture.array
	camera.close()

    #serie di filtri applicati all'immagine scattata per mettere in risalto il contorno della "tabella" di gioco (così dovrebbe funzionare in più situazioni di luce ma potrebbe essere necessaria una modifica e un miglioramento per funzionare in tutte condizioni di luminosità)
	blur = img
	#blur = cv2.GaussianBlur(img,(5,5),0)
	gray = cv2.cvtColor(blur,cv2.COLOR_BGR2GRAY)
	gray2 = cv2.adaptiveThreshold(gray,255,cv2.ADAPTIVE_THRESH_GAUSSIAN_C,cv2.THRESH_BINARY_INV,11,2)
	#ret,gray = cv2.threshold(gray,80,255,0)
	#gray2 = cv2.bitwise_not(gray)

    #trova il contorno della tabella
	cnts = cv2.findContours(gray2, cv2.RETR_LIST, cv2.CHAIN_APPROX_SIMPLE)
	cnts = imutils.grab_contours(cnts)
	cnts = sorted(cnts, key = cv2.contourArea, reverse = True)[:5]

	for c in cnts:

		peri = 0.01*cv2.arcLength(c, True)
		approx = cv2.approxPolyDP(c, peri, True)

		if len(approx) == 4:
			screenCnt = approx
	        break

    print screenCnt #di debug, se non stampati 4 valori allora probabilmente la luce non è ottimale o occorre modificare i valori dei filtri (sarebbero i 4 angoli del contorno della tabella)


    #ritaglia la tabella di gioco dall'immagine scattata e la salva, così si ha un'immagione solo della tabella senza elementi di contorno
	box = four_point_transform(img, screenCnt.reshape(4, 2))

    #qualche filtro per far risaltare i possibili simboli presenti nelle singole celle della tabella (sicuramente da migliorare, in alcuni casi per colpa di una diversa luminosità ambientale alcune celle venivano viste vuote quando contenenti un simbolo, oppure veniva confuso un simbolo con un altro (simbolo = X o O), si valuti la sostituzione della funzione threshold con la funzione adaptiveThreshold che è più "flessibile")
	warped = cv2.cvtColor(box,cv2.COLOR_BGR2GRAY)
	ret,thresh = cv2.threshold(warped,55,255,0)
	box = img.copy()

    #misurazione delle dimensioni in pixel che compongono la tabella
	height, width = thresh.shape[:2]

    #sapendo che adesso l'immagine in nostro possesso è un quadrato (la tabella) formato da 9 quadrati più piccoli (le celle) e conoscendo le dimensioni del quadrato grande, possiamo "estrarre" i quadrati più piccoli (che a noi interesano per determinare i simboli sulla tabella di gioco) semplicemente ritagliando l'immagine in 9 parti uguali e salvando ogni risultato, così abbiamo un'immagine di ogni cella singola
	start_row, start_col = int(0), int(0)
	end_row, end_col = int(height * .33), int(width * .33)
	box1 = thresh[start_row:end_row , start_col:end_col]

	start_row, start_col = int(0), int(width * .33)
	end_row, end_col = int(height * .33), int(width * .66)
	box2 = thresh[start_row:end_row , start_col:end_col]

	start_row, start_col = int(0), int(width * .66)
	end_row, end_col = int(height * .33), int(width)
	box3 = thresh[start_row:end_row , start_col:end_col]

	start_row, start_col = int(height * .33), int(0)
	end_row, end_col = int(height * .66), int(width * .33)
	box4 = thresh[start_row:end_row , start_col:end_col]

	start_row, start_col = int(height * .33), int(width * .33)
	end_row, end_col = int(height * .66), int(width * .66)
	box5 = thresh[start_row:end_row , start_col:end_col]

	start_row, start_col = int(height * .33), int(width * .66)
	end_row, end_col = int(height * .66), int(width)
	box6 = thresh[start_row:end_row , start_col:end_col]

	start_row, start_col = int(height * .66), int(0)
	end_row, end_col = int(height), int(width * .33)
	box7 = thresh[start_row:end_row , start_col:end_col]

	start_row, start_col = int(height * .66), int(width * .33)
	end_row, end_col = int(height), int(width * .66)
	box8 = thresh[start_row:end_row , start_col:end_col]

	start_row, start_col = int(height * .66), int(width * .66)
	end_row, end_col = int(height), int(width)
	box9 = thresh[start_row:end_row , start_col:end_col]

    #crea una lista con i simboli della tabella identificati con la funzione riconosci_parte
	parti = [riconosci_parte(box1),riconosci_parte(box2),riconosci_parte(box3),
	         riconosci_parte(box4),riconosci_parte(box5),riconosci_parte(box6),
			 riconosci_parte(box7),riconosci_parte(box8),riconosci_parte(box9)]

	return parti


