import random
from camera import get_board #locale
from move import move_to #locale
from gpiozero import Button
from time import sleep


#Serie di funzioni utilizzate dall'algoritmo minimax per calcolare la migliore mossa

class TicTacToe:

    def __init__(self):
        """leggi la tabella dalla fotocamera e salvala"""
        self.board = get_board()
	sleep(10)

    def show(self):
        """Scrivi la tabella"""
        print("""
          {} | {} | {}
         -----------
          {} | {} | {}
         -----------
          {} | {} | {}
        """.format(*self.board))

    def whoWon(self):
        if self.checkWin() == "X":
            return "X"
        elif self.checkWin() == "O":
            return "O"
        elif self.gameOver() == True:
            return "Nobody"

    def availableMoves(self):
        """Return empty spaces on the board"""
        moves = []
        for i in range(0, len(self.board)):
            if self.board[i] == " ":
                moves.append(i)
        return moves

    def getMoves(self, player):
        """Get all moves made by a given player"""
        moves = []
        for i in range(0, len(self.board)):
            if self.board[i] == player:
                moves.append(i)
        return moves

    def makeMove(self, position, player):
        """Make a move on the board"""
        self.board[position] = player

    def checkWin(self):
        """Return the player that wins the game"""
        combos = ([0, 1, 2], [3, 4, 5], [6, 7, 8],
                  [0, 3, 6], [1, 4, 7], [2, 5, 8],
                  [0, 4, 8], [2, 4, 6])

        for player in ("X", "O"):
            positions = self.getMoves(player)
            for combo in combos:
                win = True
                for pos in combo:
                    if pos not in positions:
                        win = False
                if win:
                    return player

    def gameOver(self):
        """Return True if X wins, O wins, or draw, else return False"""
        if self.checkWin() != None:
            return True
        for i in self.board:
            if i == " ":
                return False
        return True

    def minimax(self, node, depth, player):
        """
        Recursively analyze every possible game state and choose
        the best move location.

        node - the board
        depth - how far down the tree to look
        player - what player to analyze best move for (currently setup up ONLY for "O")
        """
        if depth == 0 or node.gameOver():
            if node.checkWin() == "X":
                return 0
            elif node.checkWin() == "O":
                return 100
            else:
                return 50

        if player == "O":
            bestValue = 0
            for move in node.availableMoves():
                node.makeMove(move, player)
                moveValue = self.minimax(node, depth-1, changePlayer(player))
                node.makeMove(move, " ")
                bestValue = max(bestValue, moveValue)
            return bestValue

        if player == "X":
            bestValue = 100
            for move in node.availableMoves():
                node.makeMove(move, player)
                moveValue = self.minimax(node, depth-1, changePlayer(player))
                node.makeMove(move, " ")
                bestValue = min(bestValue, moveValue)
            return bestValue

def changePlayer(player):
    """Returns the opposite player given any player"""
    if player == "X":
        return "O"
    else:
        return "X"

def make_best_move(board, depth, player):
    """
    Controllor function to initialize minimax and keep track of optimal move choices

    board - what board to calculate best move for
    depth - how far down the tree to go
    player - who to calculate best move for (Works ONLY for "O" right now)
    """
    neutralValue = 50
    choices = []
    for move in board.availableMoves():
        board.makeMove(move, player)
        moveValue = board.minimax(board, depth-1, changePlayer(player))
        board.makeMove(move, " ")

        if moveValue > neutralValue:
            choices = [move]
            break
        elif moveValue == neutralValue:
            choices.append(move)
    print("Scelta: ", choices)

    if len(choices) > 0:
        return random.choice(choices)
    else:
        return random.choice(board.availableMoves())


#main

#per alleggerire il carico di lavoro il nostro raspberry poco prestante "legge" la tabella solo quando viene premuto il tasto

button = Button(4) #gpio 4, pin 7 

#aspetta pressione tasto
print "Pronto! sei le X"
while button.is_pressed:
	sleep(0.1)

#inizializzazione
game = TicTacToe()

#mostra la tabella locale salvata
game.show()

while game.gameOver() == False: #finche la partita non finisce

    #aspetta che venga premuto il tasto
    print "In attesa della mossa..."
    while button.is_pressed:
        sleep(0.1)

    #aggiorna la tabella (board) attraverlo la fotocamera
    game.board = get_board()

    #mostra la tabella locale salvata
    game.show()

    if game.gameOver() == True:
        break

    print("Scelta della mossa...")
ai_move = make_best_move(game, 2, "O") #Modificare 2 se si vuole modificare "l'intelligenza", 1=stupido 2=medio 3=imbattibile (non testato)
    
    #muove il pezzo tramite braccio
    move_to(ai_move+1)
    
    #scrive la mossa sulla tabella locale
    game.makeMove(ai_move, "O")
    
    #mostra la tabella locale salvata
    game.show()

print("Game Over. " + game.whoWon() + " Vince")
